@@ -153,9 +153,13 @@ enum
 {
   PROP_0,
 
-  PROP_NAME
+  PROP_NAME,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 G_DEFINE_TYPE (ClutterBindingPool, clutter_binding_pool, G_TYPE_OBJECT);
 
 static guint
@@ -313,11 +317,12 @@ clutter_binding_pool_class_init (ClutterBindingPoolClass *klass)
    * Since: 1.0
    */
   pspec = g_param_spec_string ("name",
-                               "Name",
-                               "The unique name of the binding pool",
+                               P_("Name"),
+                               P_("The unique name of the binding pool"),
                                NULL,
                                CLUTTER_PARAM_READWRITE |
                                G_PARAM_CONSTRUCT_ONLY);
+  obj_props[PROP_NAME] = pspec;
   g_object_class_install_property (gobject_class, PROP_NAME, pspec);
 }
 
@@ -503,7 +508,7 @@ clutter_binding_pool_install_action (ClutterBindingPool  *pool,
     {
       GClosureMarshal marshal;
 
-      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
+      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_FLAGS;
       g_closure_set_marshal (closure, marshal);
     }
 
@@ -569,7 +574,7 @@ clutter_binding_pool_install_closure (ClutterBindingPool  *pool,
     {
       GClosureMarshal marshal;
 
-      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
+      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_FLAGS;
       g_closure_set_marshal (closure, marshal);
     }
 
@@ -637,7 +642,7 @@ clutter_binding_pool_override_action (ClutterBindingPool  *pool,
     {
       GClosureMarshal marshal;
 
-      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
+      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_FLAGS;
       g_closure_set_marshal (closure, marshal);
     }
 }
@@ -697,7 +702,7 @@ clutter_binding_pool_override_closure (ClutterBindingPool  *pool,
     {
       GClosureMarshal marshal;
 
-      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_ENUM;
+      marshal = _clutter_marshal_BOOLEAN__STRING_UINT_FLAGS;
       g_closure_set_marshal (closure, marshal);
     }
 }
