@@ -118,6 +118,8 @@ struct _ClutterStagePrivate
 
   gint                picks_per_frame;
 
+  ClutterPickMode     pick_buffer_mode;
+
   guint redraw_pending         : 1;
   guint is_fullscreen          : 1;
   guint is_cursor_visible      : 1;
@@ -224,10 +226,6 @@ clutter_stage_allocate (ClutterActor           *self,
   height = clutter_actor_box_get_height (box);
   _clutter_stage_window_get_geometry (priv->impl, &geom);
 
-  /* XXX: Until Cogl becomes fully responsible for backend windows Clutter
-   * need to manually keep it informed of the current window size */
-  _cogl_onscreen_clutter_backend_set_size (geom.width, geom.height);
-
   /* if the stage is fixed size (for instance, it's using a frame-buffer)
    * then we simply ignore any allocation request and override the
    * allocation chain.
@@ -299,6 +297,14 @@ clutter_stage_allocate (ClutterActor           *self,
       klass->allocate (self, &override, flags);
     }
 
+  /* XXX: Until Cogl becomes fully responsible for backend windows
+   * Clutter need to manually keep it informed of the current window
+   * size. We do this after the allocation above so that the stage
+   * window has a chance to update the window size based on the
+   * allocation. */
+  _clutter_stage_window_get_geometry (priv->impl, &geom);
+  _cogl_onscreen_clutter_backend_set_size (geom.width, geom.height);
+
   clutter_actor_get_allocation_geometry (self, &geom);
   _clutter_stage_set_viewport (CLUTTER_STAGE (self),
                                0, 0, geom.width, geom.height);
@@ -403,10 +409,6 @@ clutter_stage_realize (ClutterActor *self)
    */
   if (is_realized)
     {
-    CLUTTER_ACTOR_SET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
-    clutter_stage_ensure_current (CLUTTER_STAGE (self));
-
-
       ClutterBackend *backend = clutter_get_default_backend ();
       GError *error = NULL;
 
@@ -415,11 +417,7 @@ clutter_stage_realize (ClutterActor *self)
          Cogl functions. Otherwise it would create the Cogl context
          before we get a chance to check whether the GL version is
          valid */
-#ifndef MAEMO
-//With GLES there are no need to check GL version
-//http://bugzilla.clutter-project.org/show_bug.cgi?id=2426
       _clutter_backend_ensure_context_internal (backend, CLUTTER_STAGE (self));
-#endif
 
       /* Make sure Cogl can support the driver */
       if (!_cogl_check_driver_valid (&error))
@@ -753,7 +751,7 @@ clutter_stage_real_queue_redraw (ClutterActor *actor,
    * state changes that affects painting *or* picking so we can use
    * this point to invalidate any currently cached pick buffer.
    */
-  _clutter_stage_set_pick_buffer_valid (stage, FALSE);
+  _clutter_stage_set_pick_buffer_valid (stage, FALSE, -1);
 
   /* If the backend can't do anything with redraw clips (e.g. it already knows
    * it needs to redraw everything anyway) then don't spend time transforming
@@ -1474,7 +1472,7 @@ clutter_stage_init (ClutterStage *self)
   _clutter_stage_window_get_geometry (priv->impl, &geom);
   _clutter_stage_set_viewport (self, 0, 0, geom.width, geom.height);
 
-  _clutter_stage_set_pick_buffer_valid (self, FALSE);
+  _clutter_stage_set_pick_buffer_valid (self, FALSE, CLUTTER_PICK_ALL);
   _clutter_stage_reset_picks_per_frame_counter (self);
 }
 
@@ -2278,7 +2276,7 @@ clutter_stage_get_key_focus (ClutterStage *stage)
  *
  * Gets whether the depth cueing effect is enabled on @stage.
  *
- * Return value: %TRUE if the the depth cueing effect is enabled
+ * Return value: %TRUE if the depth cueing effect is enabled
  *
  * Since: 0.6
  */
@@ -2955,20 +2953,25 @@ clutter_stage_get_no_clear_hint (ClutterStage *stage)
 }
 
 gboolean
-_clutter_stage_get_pick_buffer_valid (ClutterStage *stage)
+_clutter_stage_get_pick_buffer_valid (ClutterStage *stage, ClutterPickMode mode)
 {
   g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
 
+  if (stage->priv->pick_buffer_mode != mode)
+    return FALSE;
+
   return stage->priv->have_valid_pick_buffer;
 }
 
 void
-_clutter_stage_set_pick_buffer_valid (ClutterStage *stage,
-                                      gboolean      valid)
+_clutter_stage_set_pick_buffer_valid (ClutterStage   *stage,
+                                      gboolean        valid,
+                                      ClutterPickMode mode)
 {
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
 
   stage->priv->have_valid_pick_buffer = !!valid;
+  stage->priv->pick_buffer_mode = mode;
 }
 
 void
