@@ -698,6 +698,10 @@ _cogl_pot_slices_for_size (int    size_to_fill,
       else if (span.size - size_to_fill <= max_waste)
         {
           /* Yes and waste is small enough */
+          /* Pick the next power of two up from size_to_fill. This can
+             sometimes be less than the span.size that would be chosen
+             otherwise */
+          span.size = _cogl_util_next_p2 (size_to_fill);
           span.waste = span.size - size_to_fill;
           if (out_spans)
             g_array_append_val (out_spans, span);
@@ -783,8 +787,8 @@ _cogl_texture_2d_sliced_slices_create (CoglTexture2DSliced *tex_2ds,
     }
   else
     {
-      max_width = cogl_util_next_p2 (width);
-      max_height = cogl_util_next_p2 (height);
+      max_width = _cogl_util_next_p2 (width);
+      max_height = _cogl_util_next_p2 (height);
       tex_2ds->gl_target = GL_TEXTURE_2D;
       slices_for_size = _cogl_pot_slices_for_size;
     }
@@ -1052,17 +1056,6 @@ _cogl_texture_2d_sliced_new_with_size (unsigned int     width,
       return COGL_INVALID_HANDLE;
     }
 
-  if (!_cogl_texture_2d_sliced_slices_create (tex_2ds,
-                                              width,
-                                              height,
-                                              gl_intformat,
-                                              gl_format,
-                                              gl_type))
-    {
-      _cogl_texture_2d_sliced_free (tex_2ds);
-      return COGL_INVALID_HANDLE;
-    }
-
   tex_2ds->auto_mipmap = (flags & COGL_TEXTURE_NO_AUTO_MIPMAP) == 0;
 
   return _cogl_texture_2d_sliced_handle_new (tex_2ds);
@@ -1121,13 +1114,6 @@ _cogl_texture_2d_sliced_new_from_bitmap (CoglBitmap      *bmp,
                                           internal_format))
     goto error;
 
-  if (!_cogl_texture_2d_sliced_slices_create (tex_2ds,
-                                              width, height,
-                                              gl_intformat,
-                                              gl_format,
-                                              gl_type))
-    goto error;
-
   if (!_cogl_texture_2d_sliced_upload_to_gl (tex_2ds,
                                              dst_bmp,
                                              gl_intformat,
