@@ -132,6 +132,8 @@ clutter_backend_egl_post_parse (ClutterBackend  *backend,
   return TRUE;
 }
 
+static gboolean first_run=TRUE;
+
 static gboolean
 clutter_backend_egl_create_context (ClutterBackend  *backend,
                                     GError         **error)
@@ -147,18 +149,15 @@ clutter_backend_egl_create_context (ClutterBackend  *backend,
     /* NB: This must be the first attribute, since we may
      * try and fallback to no stencil buffer */
     EGL_STENCIL_SIZE,    2,
-#ifndef MAEMO
+
     EGL_RED_SIZE,        1,
     EGL_GREEN_SIZE,      1,
     EGL_BLUE_SIZE,       1,
-    EGL_BUFFER_SIZE,     EGL_DONT_CARE,
-#else
+    EGL_ALPHA_SIZE,      EGL_DONT_CARE,
 
-   EGL_BUFFER_SIZE,    16,
-#endif
-    EGL_ALPHA_SIZE,    EGL_DONT_CARE,
     EGL_DEPTH_SIZE,      1,
 
+    EGL_BUFFER_SIZE,     EGL_DONT_CARE,
 
 #if defined (HAVE_COGL_GL)
     EGL_RENDERABLE_TYPE, EGL_OPENGL_BIT,
@@ -349,6 +348,8 @@ retry:
    * current immediately to make sure the Cogl and Clutter
    * can query the EGL context for features.
    */
+if (first_run) first_run=FALSE, //MAEMO5
+
   status = eglMakeCurrent (backend_egl->edpy,
                            backend_egl->egl_surface,
                            backend_egl->egl_surface,
